# MT::Plugins::Redland::CMS::Topic
# -----------------
# $Revision: 1.7 $
# $Date: 2005/04/11 05:36:02 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic;

use strict;
use warnings;

use Carp qw(confess);
use Encode;
use IO::All;
use Data::Dumper;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME			= "Redland Entry Topic";
	$VERSION		= do { my @REV = split(/\./, (qw$Revision: 1.7 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug			= 0;
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds foaf:topic statements to an Entry',
		doc_link	=> 'http://kasei.us/code/mtsemweb/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms.pl' );
	require $cms;
}

######################################################################

my $cfg		= MT::ConfigMgr->instance;
if ($cfg->ObjectDriver eq 'Redland') {
	MT::Plugins::Redland::CMS->install_cms_header( $NAME, sub {
		my $app		= shift;
		my $q		= $app->{query};
		my $mode	= $q->param('__mode') || '';
		my $type	= $q->param('_type') || '';
		if ($type eq 'entry' and $mode eq 'view') {
			my $cfg		= MT::ConfigMgr->instance;
			my $path	= $cfg->StaticWebPath || $app->path;
			return qq(<script type="text/javascript" src="${path}easytoggle3.js"></script>);
		}
	} );
	
	MT::Plugins::Redland::CMS->install_cms_box( $NAME, sub {
		my $app		= shift;
		my $q		= $app->{query};
		my $mode	= $q->param('__mode') || '';
		my $type	= $q->param('_type') || '';
		if ($type eq 'entry' and $mode eq 'view') {
			my @first;
			my @rest;
			my $count	= 1;
			our %topics;
			while (my($k,$v) = each(%topics)) {
				my $body	= ref($v) ? $v->( $app ) : $v;
				push(@first, qq(<li><a class="toggle" title="${k}" href="#${k}">${k}</a></li>));
				push(@rest, qq(<div id="${k}">${body}</div>));
			} continue { $count++ }
			
			my $body	= qq(<div><label>Entry Topic</label></div>\n)
						. qq(<div id="toggle"><ul class="toggle">)
						. join("\n", @first)
						. qq(</ul></div>\n)
						. qq(<div id="togglebox">) . join("\n", @rest) . qq(</div>\n);
			return $body;
		}
	} );
	
	##################################################################
	
	MT::Template::Context->add_container_tag( IfEntryTopic => sub {
		my $ctx		= shift;
		my $args	= shift;
		my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
		my $model	= $entry->driver->model;
		my $node	= $entry->get_rdf_node;
		
		my $topic;
		if (my $type = $args->{'type'}) {
			$topic		= $entry->rdf_topic_of_type( $type );
		} else {
			($topic)	= $entry->rdf_topics();
		}
		
		my $res		= '';
		if ($topic) {
			$ctx->stash('rdf_topic', $topic);
			my $builder	= $ctx->stash('builder');
			my $tokens	= $ctx->stash('tokens');
			defined(my $out = $builder->build($ctx, $tokens)) or return $ctx->error($builder->errstr);
			$ctx->stash('rdf_topic', undef);
			$res		.= $out;
		}
		$res;
	} );

	MT::Template::Context->add_container_tag( IfNotEntryTopic => sub {
		my $ctx		= shift;
		my $args	= shift;
		my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
		my $model	= $entry->driver->model;
		my $node	= $entry->get_rdf_node;
		
		my $not		= 1;
		if (my $type = $args->{'type'}) {
			my $topic	= $entry->rdf_topic_of_type( $type );
			$not		= 0 if ($topic);
		} else {
			my ($topic)	= $entry->rdf_topics();
			$not		= 0 if ($topic);
		}
		
		my $res		= '';
		if ($not) {
			my $builder	= $ctx->stash('builder');
			my $tokens	= $ctx->stash('tokens');
			defined(my $out = $builder->build($ctx, $tokens)) or return $ctx->error($builder->errstr);
			$res		.= $out;
		}
		$res;
	} );

	MT::Template::Context->add_tag( EntryTopicHTML => sub {
		my $warnings;
		local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
		my $ctx		= shift;
		my $args	= shift;
		my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
		my $model	= $entry->driver->model;
		my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topics();
		if ($topic) {
			my $label	= get_first_literal( $model, $topic, ['http://purl.org/dc/elements/1.1/title', 'http://www.w3.org/2000/01/rdf-schema#label', 'http://purl.org/dc/elements/1.1/description'] );
			if ($label) {
				if ($args->{maxlength} and length($label) > $args->{maxlength}) {
					$label	= substr($label, 0, $args->{maxlength} - 1) . '&#8230;';
				}
			} else {
				$label	= $topic->uri->as_string if ($topic->is_resource);
				if ($args->{maxlength} and length($label) > $args->{maxlength}) {
					$label	= substr($label, rindex($label,'/')+1);
				}
			}
			my $data	= '';
			
			if ($label) {
				my $link	= get_first_uri($model, $topic, 'http://xmlns.com/foaf/0.1/page');
				$link		||= $topic->is_resource ? $topic->uri->as_string : undef;
				$data	= ($link) ? qq(<a href="${link}">${label}</a>) : $label;
				return $data . "\n<!--\n$warnings\n-->\n" if ($debug);
				return $data;
			}
		}
		
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	} );

	MT::Template::Context->add_tag( EntryTopicURL => sub {
		my $warnings;
		local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
		my $ctx		= shift;
		my $args	= shift;
		my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
		my $model	= $entry->driver->model;
		my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topics();
		if ($topic and $topic->is_resource) {
			my $data	= $topic->uri->as_string;
			return $data;
		} else {
			return "\n<!--\n$warnings\n-->\n" if ($debug);
			return '';
		}
	} );

	MT::Template::Context->add_tag( EntryTopic => sub {
		my $warnings;
		local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
		my $ctx		= shift;
		my $args	= shift;
		my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
		my $model	= $entry->driver->model;
		my $node	= $entry->get_rdf_node;
		my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topics();
		my $pred	= $args->{'attribute'} || $args->{'attr'} || $args->{'pred'};
		if ($topic and $pred) {
			my $p		= RDF::Redland::Node->new_from_uri( $pred );
			my ($val)	= $model->targets( $topic, $p );
			my $data	= '';
			if ($val) {
				if ($val->is_resource) {
					$data	= $val->uri->as_string;
				} elsif ($val->is_literal) {
					$data	= decode('utf8', $val->literal_value);
				} else {
					$data	= $val->blank_identifier;
				}
			}
			return $data . "\n<!--\n$warnings\n-->\n" if ($debug);
			return $data;
		} else {
			return "\n<!--\n$warnings\n-->\n" if ($debug);
			return '';
		}
	} );
}

######################################################################

sub install_topic {
	my $class	= shift;
	my $name	= shift;
	my $body	= shift;
	our %topics;
	$topics{ $name }	= $body;
}

sub MT::Entry::rdf_topic_of_type {
	my $entry	= shift;
	my $type	= shift;
	$type		= RDF::Redland::Node->new_from_uri( $type ) unless ref($type);
	my @topics	= $entry->rdf_topics;
	my $model	= $entry->driver->model;
	my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	foreach my $topic (@topics) {
		warn "topic: " . $topic->as_string if ($debug);
		my $st	= RDF::Redland::Statement->new( $topic, $p_type, $type );
		warn $st->as_string if ($debug);
		if ($model->contains_statement( $st )) {
			return $topic;
		}
	}
	
}

sub MT::Entry::rdf_topics {
	my $entry	= shift;
	my $model	= $entry->driver->model;
	my $node	= $entry->get_rdf_node;
	my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
	my @topics	= $model->targets( $node, $p_topic );
	return @topics;
}

sub MT::Entry::remove_rdf_topics_of_type {
	my $entry	= shift;
	my $type	= shift;
	my $node	= $entry->get_rdf_node;
	my $model	= $entry->driver->model;
	my @topics	= $entry->rdf_topics();
	my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
	
	foreach my $topic (@topics) {
		my @types	= $model->targets( $topic, $p_type );
		foreach my $topictype (@types) {
			if ($type->equals( $topictype )) {
				my $st		= RDF::Redland::Statement->new( $node, $p_topic, $topic );
				$model->remove_statement( $st );
				last;
			}
		}
	}
}

sub get_first_literal {
	my $node	= get_first_obj( @_ );
	return $node ? decode('utf8', $node->literal_value) : undef;
}

sub get_all_literal {
	my @nodes	= get_all_obj( @_ );
	return map { decode('utf8', $_->literal_value) } grep { $_->can('literal_value') } @nodes;
}

sub get_first_uri {
	my $node	= get_first_obj( @_ );
	return $node ? $node->uri->as_string : undef;
}

sub get_all_uri {
	my @nodes	= get_all_obj( @_ );
	return map { $_->uri->as_string } grep { defined($_) and $_->uri } @nodes;
}

sub get_first_obj {
	my $model	= shift;
	my $node	= shift;
	my $uri		= shift;
	my @uris	= UNIVERSAL::isa($uri, 'ARRAY') ? @{ $uri } : ($uri);
	my @preds	= map { ref($_) ? $_ : RDF::Redland::Node->new_from_uri( $_ ) } @uris;
	foreach my $pred (@preds) {
		my $targets	= $model->targets_iterator( $node, $pred );
		while ($targets and !$targets->end) {
			my $node	= $targets->current;
			return $node if ($node);
		} continue { $targets->next }
	}
}

sub get_all_obj {
	my $model	= shift;
	my $node	= shift;
	my $uri		= shift;
	my @uris	= UNIVERSAL::isa($uri, 'ARRAY') ? @{ $uri } : ($uri);
	my @preds	= map { ref($_) ? $_ : RDF::Redland::Node->new_from_uri( $_ ) } @uris;
	my @objs;
	return map { $model->targets( $node, $_ ) } @preds;
}


1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic.pl,v $
 Revision 1.7  2005/04/11 05:36:02  greg
 - added new template tags: IfEntryTopic, IfNotEntryTopic, EntryTopicHTML, EntryTopicURL, EntryTopic

 Revision 1.6  2005/04/06 15:09:45  greg
 - bumped version number to 0.005
 - updated email address in AUTHOR POD section

 Revision 1.5  2005/04/05 05:35:03  greg
 - updated docs
 - now only addes CMS hooks if Redland is the ObjectDriver

 Revision 1.4  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.3  2005/04/04 07:46:37  greg
 - moved topic_of_type() and topics() into MT::Entry namespace as
   rdf_topic_of_type() and rdf_topics()

 Revision 1.2  2005/04/03 22:19:35  greg
 - topic CMS hooks now only run if on the edit entry page
 - added a foaf:Document topic plugin

 Revision 1.1  2005/04/03 01:07:41  greg
 - added redland CMS Topic plugin
 - moved redland_cms_book.pl to redland_cms_topic_book.pl, now uses Topic API
 - added redland_cms_topic_event.pl for Event topics


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

=over 4

=item * Make Topic tag selected if there's an existing topic

=back

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <greg@evilfunhouse.com>

=cut
